import asyncio


from spotON_sdk import API_Call,Timeframes,Price_Logic,Markets,Continuous_On_Time,request_Day_Ahead_Price


from pricelogic_Test import price_Logic1

from spotON_helper.authentification.expireing_Token import generate_token
API_token = generate_token("test")



timeframes = Timeframes()
timeframes.add_timeframe(1,5)
price_Logic1 = Price_Logic(nr_of_hours_on=10,market=Markets.germany,timeframes=timeframes,pricefinding=Continuous_On_Time(week=1,best_hour=1))
#apicode = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VyX2lkIjoiMTIzNCIsImV4cCI6MTY4NTU3MzAwOH0.hqPhb7IvpB53qP3NRKkWEd3cp4qtWlND1cVcjTCo-zg"
apicode = generate_token("test")

data = API_Call(spotOn_API_code=apicode,price_logic=price_Logic1)


asyncio.run(request_Day_Ahead_Price(data))


from dataclasses import dataclass,field


@dataclass
class Unit:
    name: str
    unit: str

class Units():
    temperature = Unit("Temperature","°C")

@dataclass
class Humidity(Unit):
    name = "Humidity"
    unit = "%"

@dataclass
class Co2(Unit):
    name = "Co2"
    unit = "ppm"

@dataclass
class Pressure(Unit):
    name = "Pressure"
    unit = "P"


@dataclass
class Volt(Unit):
    name = "Measurement"
    unit = "mV"


import pandas as pd

def preprocess(df,timezone:str):
    #TODO: modify so that it works with more days
    df.index =  pd.to_datetime(df.index, utc=True)
    df = df.rename_axis('DateTime')
    df = df.rename(columns={"value":"Price"})
    
    # Write the week number and day number into the DataFrame in Local Time
    df_tz = df.tz_convert(timezone)
    df['Week'] = df_tz.index.isocalendar().week
    df['Day'] = df_tz.index.day
    df["Hour"] = df_tz.index.hour
    print (f"{df=}")
    # Calculate the daily average price and store it in a new DataFrame
    average_price = df['Price'].mean()

    df['Price_daily_AVG'] = average_price
    
    return df

def add_priceRating(df):
    df['rating'] = df['Price'] - df['Price_daily_AVG']
    df['rating_rank'] = df.groupby(df.index.date)['rating'].rank(ascending=True)-1
    return df

def add_priceRatings(df):
    df['savings'] = df['Price'] - df['Price_daily_AVG']
    df['rating_rank'] = df.groupby(df.index.date)['savings'].rank(ascending=True)-1
    return df

def add_timeslot_savings(df):
    df['savings'] = df['Price'] - df['Price_daily_AVG']
    for timeslot_length in range(1,24):
        df[f"timeslot_{timeslot_length}H"] = df["savings"].rolling(timeslot_length).mean().shift(-timeslot_length+1)
    return df


def reduce_Filesize(df:pd.DataFrame):
    df["Price"] = df["Price"].astype('float16')
    df["Week"] = df["Week"].astype("int8")
    df["Day"] = df["Day"].astype("int8")
    df["Hour"] = df["Hour"].astype("int8")
    df["Price_daily_AVG"] = df["Price_daily_AVG"].astype('float16')
    df["savings"] = df["savings"].astype('float16')
    for timeslot_length in range(1,24):
        df[f"timeslot_{timeslot_length}H"] = df[f"timeslot_{timeslot_length}H"].astype('float16')
    return df




from typing import List, Tuple
from dataclasses import dataclass
import json
import base64

from pricelogic_Test import price_Logic1
from spotON_sdk import Market,Price_Logic

@dataclass()
class SwitchPattern:
    area_code :str
    nr_of_hours_on :int
    pricefinding :dict
    resolution :float
    timeframes : List[int]

def encode_switch_pattern(switch_pattern: SwitchPattern) -> str:
    switch_pattern_str = json.dumps(switch_pattern.__dict__)
    switch_pattern_bytes = switch_pattern_str.encode('utf-8')
    switch_pattern_b64 = base64.b64encode(switch_pattern_bytes)
    return switch_pattern_b64.decode('utf-8')

def decode_switch_pattern(encoded_switch_pattern: str) -> SwitchPattern:
    switch_pattern_bytes = base64.b64decode(encoded_switch_pattern)
    switch_pattern_str = switch_pattern_bytes.decode('utf-8')
    switch_pattern_dict = json.loads(switch_pattern_str)
    return SwitchPattern(**switch_pattern_dict)

def convert_switch_pattern(price_Logic1: Price_Logic):
        
    area_code = price_Logic1.market.area_code # type: ignore
    nr_of_hours_on = price_Logic1.nr_of_hours_on
    pricefinding = price_Logic1.pricefinding.__dict__
    resolution = price_Logic1.resolution
    timeframes = price_Logic1.timeframes.possible_hours

    # Create and return a new SwitchPattern object
    return SwitchPattern(area_code, nr_of_hours_on, pricefinding, resolution, timeframes)

def debug():

    switch_pattern = convert_switch_pattern(price_Logic1)
    print(switch_pattern)

    # Encode the switch pattern
    switch_pattern_encoded = encode_switch_pattern(switch_pattern)
    print(switch_pattern_encoded)
    

    # Decode the switch pattern
    switch_pattern_decoded = decode_switch_pattern(switch_pattern_encoded)
    print(switch_pattern_decoded)

    # Verify that the decoded switch pattern matches the original
    assert switch_pattern == switch_pattern_decoded



switch_pattern = convert_switch_pattern(price_Logic1)
switch_pattern_encoded = encode_switch_pattern(switch_pattern)
print(f"{switch_pattern_encoded=}")



import pandas as pd
import pytz

class Timestamps_for_Testing:
    ''' get a timestamp for a special timezones and dates for testing purposes
    
    usage example:timestamp = Timestamps_for_Testing("Europe/Berlin").day_before_summer_winter_timechange_end
    returns a timestamp for the day before the summer winter timechange end in Berlin

    
    this line adds all edge cases for the timestamps:
    summer_winter_timechange_start,summer_winter_timechange_end,day_before_summer_winter_timechange_start,day_before_summer_winter_timechange_end,winter_summer_timechange_start,winter_summer_timechange_end,day_before_winter_summer_timechange_start,day_before_winter_summer_timechange_end,normalday_start,normalday_end = Timestamps_for_Testing("Europe/Berlin").return_all_timestamps() 
    '''

    def __init__(self,timezone :str) -> None:
        self.timezone = timezone
        self.summer_winter_timechange_start = pd.Timestamp(2023,10,29,0,0,0).tz_localize(timezone)
        self.summer_winter_timechange_end = pd.Timestamp(2023,10,29,23,0,0).tz_localize(timezone)
        self.day_before_summer_winter_timechange_start = pd.Timestamp(2023,10,28,0,0,0).tz_localize(timezone)
        self.day_before_summer_winter_timechange_end = pd.Timestamp(2023,10,28,23,0,0).tz_localize(timezone)
        self.winter_summer_timechange_start = pd.Timestamp(2021,3,28,0,0,0).tz_localize(timezone)
        self.winter_summer_timechange_end = pd.Timestamp(2021,3,28,23,0,0).tz_localize(timezone)
        self.day_before_winter_summer_timechange_start = pd.Timestamp(2021,3,27,0,0,0).tz_localize(timezone)
        self.day_before_winter_summer_timechange_end  = pd.Timestamp(2021,3,27,23,0,0).tz_localize(timezone)

        self.normalday_start = pd.Timestamp(2021,1,1,0,0,0).tz_localize(timezone)
        self.normalday_end = pd.Timestamp(2021,1,1,23,0,0).tz_localize(timezone)

    def return_all_timestamps(self):
        return self.summer_winter_timechange_start,self.summer_winter_timechange_end,self.day_before_summer_winter_timechange_start,self.day_before_summer_winter_timechange_end,self.winter_summer_timechange_start,self.winter_summer_timechange_end,self.day_before_winter_summer_timechange_start,self.day_before_winter_summer_timechange_end,self.normalday_start,self.normalday_end





from typing import List
from dataclasses import dataclass


from spotON_sdk.Logic.Feedback import Feedback
from spotON_sdk.Logic.Output import Switchtypes
from spotON_sdk.Logic.Price import Price_Logic


@dataclass
class Controller():
    name :str
    config  : Price_Logic
    feedback : List[Feedback]
    output_Switch : Switchtypes


from enum import Enum
from dataclasses import dataclass
from pydantic import BaseModel


class Currency_unit(BaseModel):
    name: str
    symbol: str 

class Currency(BaseModel):
    majorUnit: Currency_unit
    minorUnit: Currency_unit
    iso_code_4217: str 

class Currencies:
    euro = Currency(
        majorUnit=Currency_unit(name="Euro", symbol="€"),
        minorUnit=Currency_unit(name="cent", symbol="cent"),
        iso_code_4217 = "EUR"
    )


    bulgarian_lev = Currency(
        majorUnit=Currency_unit(name="Bulgarian Lev", symbol="лв"),
        minorUnit=Currency_unit(name="stotinki", symbol="st."),
        iso_code_4217="BGN"
    )

    croatian_kuna = Currency(
        majorUnit=Currency_unit(name="Croatian Kuna", symbol="kn"),
        minorUnit=Currency_unit(name="lipa", symbol="lp"),
        iso_code_4217="HRK"
    )

    czech_koruna = Currency(
        majorUnit=Currency_unit(name="Czech Koruna", symbol="Kč"),
        minorUnit=Currency_unit(name="haléř", symbol="h"),
        iso_code_4217="CZK"
    )

    danish_krone = Currency(
        majorUnit=Currency_unit(name="Danish Krone", symbol="kr"),
        minorUnit=Currency_unit(name="øre", symbol="øre"),
        iso_code_4217="DKK"
    )

    hungarian_forint = Currency(
        majorUnit=Currency_unit(name="Hungarian Forint", symbol="Ft"),
        minorUnit=Currency_unit(name="fillér", symbol="f"),
        iso_code_4217="HUF"
    )


    polish_zloty = Currency(
        majorUnit=Currency_unit(name="Polish Złoty", symbol="zł"),
        minorUnit=Currency_unit(name="grosz", symbol="gr"),
        iso_code_4217="PLN"
    )


    romanian_leu = Currency(
        majorUnit=Currency_unit(name="Romanian Leu", symbol="lei"),
        minorUnit=Currency_unit(name="bani", symbol="b"),
        iso_code_4217="RON"
    )


    swedish_krona = Currency(
        majorUnit=Currency_unit(name="Swedish Krona", symbol="kr"),
        minorUnit=Currency_unit(name="öre", symbol="öre"),
        iso_code_4217="SEK"
    )

import pytz
from datetime import datetime

class Area_details(BaseModel):
    name: str
    code: str
    tz: str
    currency: Currency = Currencies.euro
    tz_utc_difference: int = 0

    def __init__(self, **data):
        super().__init__(**data)
        self.tz_utc_difference = self.calculate_utc_offset()

    def calculate_utc_offset(self):
        """
        Returns the UTC offset in hours for the area's time zone.
        """
        timezone = pytz.timezone(self.tz)
        # Get current UTC offset in seconds
        utc_offset_seconds = timezone.utcoffset(datetime.now()).total_seconds()
        # Convert seconds to hours
        utc_offset_hours = int(utc_offset_seconds / 3600)
        return utc_offset_hours


class spotON_Area(Enum):
    AL = Area_details(name='AL',code='10YAL-KESH-----5',tz='Europe/Tirane',)
    AT = Area_details(name='AT',code='10YAT-APG------L',tz='Europe/Vienna',currency=Currencies.euro)
    BA = Area_details(name='BA',code='10YBA-JPCC-----D',tz='Europe/Sarajevo')
    BE = Area_details(name='BE',code='10YBE----------2',tz='Europe/Brussels')
    BG = Area_details(name='BG',code='10YCA-BULGARIA-R',tz='Europe/Sofia',currency=Currencies.bulgarian_lev)
    BY = Area_details(name='BY',code='10Y1001A1001A51S',tz='Europe/Minsk')
    CH = Area_details(name='CH',code='10YCH-SWISSGRIDZ',tz='Europe/Zurich')
    CWE = Area_details(name='CWE',code='10YDOM-REGION-1V',tz='Europe/Brussels')
    CY = Area_details(name='CY',code='10YCY-1001A0003J',tz='Asia/Nicosia')
    CZ = Area_details(name='CZ',code='10YCZ-CEPS-----N',tz='Europe/Prague',currency=Currencies.czech_koruna)
    CZ_DE_SK = Area_details(name='CZ_DE_SK',code='10YDOM-CZ-DE-SKK',tz='Europe/Prague',currency=Currencies.czech_koruna)
    DE = Area_details(name='DE',code='10Y1001A1001A83F',tz='Europe/Berlin')
    DE_50HZ = Area_details(name='DE_50HZ',code='10YDE-VE-------2',tz='Europe/Berlin')
    DE_AMPRION = Area_details(name='DE_AMPRION',code='10YDE-RWENET---I',tz='Europe/Berlin')
    DE_AT_LU = Area_details(name='DE_AT_LU',code='10Y1001A1001A63L',tz='Europe/Berlin')
    DE_LU = Area_details(name='DE_LU',code='10Y1001A1001A82H',tz='Europe/Berlin')
    DE_TENNET = Area_details(name='DE_TENNET',code='10YDE-EON------1',tz='Europe/Berlin')
    DE_TRANSNET = Area_details(name='DE_TRANSNET',code='10YDE-ENBW-----N',tz='Europe/Berlin')
    DK = Area_details(name='DK',code='10Y1001A1001A65H',tz='Europe/Copenhagen',currency=Currencies.danish_krone)
    DK_1 = Area_details(name='DK_1',code='10YDK-1--------W',tz='Europe/Copenhagen',currency=Currencies.danish_krone)
    DK_1_NO_1 = Area_details(name='DK_1_NO_1',code='46Y000000000007M',tz='Europe/Copenhagen',currency=Currencies.danish_krone)
    DK_2 = Area_details(name='DK_2',code='10YDK-2--------M',tz='Europe/Copenhagen',currency=Currencies.danish_krone)
    DK_CA = Area_details(name='DK_CA',code='10Y1001A1001A796',tz='Europe/Copenhagen',currency=Currencies.danish_krone)
    EE = Area_details(name='EE',code='10Y1001A1001A39I',tz='Europe/Tallinn')
    ES = Area_details(name='ES',code='10YES-REE------0',tz='Europe/Madrid')
    FI = Area_details(name='FI',code='10YFI-1--------U',tz='Europe/Helsinki')
    FR = Area_details(name='FR',code='10YFR-RTE------C',tz='Europe/Paris')
    GB = Area_details(name='GB',code='10YGB----------A',tz='Europe/London')
    GB_ELECLINK = Area_details(name='GB_ELECLINK',code='11Y0-0000-0265-K',tz='Europe/London')
    GB_IFA = Area_details(name='GB_IFA',code='10Y1001C--00098F',tz='Europe/London')
    GB_IFA2 = Area_details(name='GB_IFA2',code='17Y0000009369493',tz='Europe/London')
    GB_NIR = Area_details(name='GB_NIR',code='10Y1001A1001A016',tz='Europe/Belfast')
    GR = Area_details(name='GR',code='10YGR-HTSO-----Y',tz='Europe/Athens')
    HR = Area_details(name='HR',code='10YHR-HEP------M',tz='Europe/Zagreb',currency=Currencies.croatian_kuna)
    HU = Area_details(name='HU',code='10YHU-MAVIR----U',tz='Europe/Budapest',currency=Currencies.hungarian_forint)
    IE = Area_details(name='IE',code='10YIE-1001A00010',tz='Europe/Dublin')
    IE_SEM = Area_details(name='IE_SEM',code='10Y1001A1001A59C',tz='Europe/Dublin')
    IS = Area_details(name='IS',code='IS',tz='Atlantic/Reykjavik')
    IT = Area_details(name='IT',code='10YIT-GRTN-----B',tz='Europe/Rome')
    IT_BRNN = Area_details(name='IT_BRNN',code='10Y1001A1001A699',tz='Europe/Rome')
    IT_CALA = Area_details(name='IT_CALA',code='10Y1001C--00096J',tz='Europe/Rome')
    IT_CNOR = Area_details(name='IT_CNOR',code='10Y1001A1001A70O',tz='Europe/Rome')
    IT_CSUD = Area_details(name='IT_CSUD',code='10Y1001A1001A71M',tz='Europe/Rome')
    IT_FOGN = Area_details(name='IT_FOGN',code='10Y1001A1001A72K',tz='Europe/Rome')
    IT_GR = Area_details(name='IT_GR',code='10Y1001A1001A66F',tz='Europe/Rome')
    IT_MACRO_NORTH = Area_details(name='IT_MACRO_NORTH',code='10Y1001A1001A84D',tz='Europe/Rome')
    IT_MACRO_SOUTH = Area_details(name='IT_MACRO_SOUTH',code='10Y1001A1001A85B',tz='Europe/Rome')
    IT_MALTA = Area_details(name='IT_MALTA',code='10Y1001A1001A877',tz='Europe/Rome')
    IT_NORD = Area_details(name='IT_NORD',code='10Y1001A1001A73I',tz='Europe/Rome')
    IT_NORD_AT = Area_details(name='IT_NORD_AT',code='10Y1001A1001A80L',tz='Europe/Rome')
    IT_NORD_CH = Area_details(name='IT_NORD_CH',code='10Y1001A1001A68B',tz='Europe/Rome')
    IT_NORD_FR = Area_details(name='IT_NORD_FR',code='10Y1001A1001A81J',tz='Europe/Rome')
    IT_NORD_SI = Area_details(name='IT_NORD_SI',code='10Y1001A1001A67D',tz='Europe/Rome')
    IT_PRGP = Area_details(name='IT_PRGP',code='10Y1001A1001A76C',tz='Europe/Rome')
    IT_ROSN = Area_details(name='IT_ROSN',code='10Y1001A1001A77A',tz='Europe/Rome')
    IT_SACO_AC = Area_details(name='IT_SACO_AC',code='10Y1001A1001A885',tz='Europe/Rome')
    IT_SACO_DC = Area_details(name='IT_SACO_DC',code='10Y1001A1001A893',tz='Europe/Rome')
    IT_SARD = Area_details(name='IT_SARD',code='10Y1001A1001A74G',tz='Europe/Rome')
    IT_SICI = Area_details(name='IT_SICI',code='10Y1001A1001A75E',tz='Europe/Rome')
    IT_SUD = Area_details(name='IT_SUD',code='10Y1001A1001A788',tz='Europe/Rome')
    LT = Area_details(name='LT',code='10YLT-1001A0008Q',tz='Europe/Vilnius')
    LU = Area_details(name='LU',code='10YLU-CEGEDEL-NQ',tz='Europe/Luxembourg')
    LV = Area_details(name='LV',code='10YLV-1001A00074',tz='Europe/Riga')
    MD = Area_details(name='MD',code='10Y1001A1001A990',tz='Europe/Chisinau')
    ME = Area_details(name='ME',code='10YCS-CG-TSO---S',tz='Europe/Podgorica')
    MK = Area_details(name='MK',code='10YMK-MEPSO----8',tz='Europe/Skopje')
    MT = Area_details(name='MT',code='10Y1001A1001A93C',tz='Europe/Malta')
    NL = Area_details(name='NL',code='10YNL----------L',tz='Europe/Amsterdam')
    NO = Area_details(name='NO',code='10YNO-0--------C',tz='Europe/Oslo')
    NO_1 = Area_details(name='NO_1',code='10YNO-1--------2',tz='Europe/Oslo')
    NO_1A = Area_details(name='NO_1A',code='10Y1001A1001A64J',tz='Europe/Oslo')
    NO_2 = Area_details(name='NO_2',code='10YNO-2--------T',tz='Europe/Oslo')
    NO_2A = Area_details(name='NO_2A',code='10Y1001C--001219',tz='Europe/Oslo')
    NO_2_NSL = Area_details(name='NO_2_NSL',code='50Y0JVU59B4JWQCU',tz='Europe/Oslo')
    NO_3 = Area_details(name='NO_3',code='10YNO-3--------J',tz='Europe/Oslo')
    NO_4 = Area_details(name='NO_4',code='10YNO-4--------9',tz='Europe/Oslo')
    NO_5 = Area_details(name='NO_5',code='10Y1001A1001A48H',tz='Europe/Oslo')
    PL = Area_details(name='PL',code='10YPL-AREA-----S',tz='Europe/Warsaw',currency=Currencies.polish_zloty)
    PL_CZ = Area_details(name='PL_CZ',code='10YDOM-1001A082L',tz='Europe/Warsaw',currency=Currencies.polish_zloty)
    PT = Area_details(name='PT',code='10YPT-REN------W',tz='Europe/Lisbon')
    RO = Area_details(name='RO',code='10YRO-TEL------P',tz='Europe/Bucharest',currency=Currencies.romanian_leu)
    RS = Area_details(name='RS',code='10YCS-SERBIATSOV',tz='Europe/Belgrade')
    RU = Area_details(name='RU',code='10Y1001A1001A49F',tz='Europe/Moscow')
    RU_KGD = Area_details(name='RU_KGD',code='10Y1001A1001A50U',tz='Europe/Kaliningrad')
    SE = Area_details(name='SE',code='10YSE-1--------K',tz='Europe/Stockholm',currency=Currencies.swedish_krona)
    SE_1 = Area_details(name='SE_1',code='10Y1001A1001A44P',tz='Europe/Stockholm',currency=Currencies.swedish_krona)
    SE_2 = Area_details(name='SE_2',code='10Y1001A1001A45N',tz='Europe/Stockholm',currency=Currencies.swedish_krona)
    SE_3 = Area_details(name='SE_3',code='10Y1001A1001A46L',tz='Europe/Stockholm',currency=Currencies.swedish_krona)
    SE_4 = Area_details(name='SE_4',code='10Y1001A1001A47J',tz='Europe/Stockholm',currency=Currencies.swedish_krona)
    SI = Area_details(name='SI',code='10YSI-ELES-----O',tz='Europe/Ljubljana')
    SK = Area_details(name='SK',code='10YSK-SEPS-----K',tz='Europe/Bratislava')
    TR = Area_details(name='TR',code='10YTR-TEIAS----W',tz='Europe/Istanbul')
    UA = Area_details(name='UA',code='10Y1001C--00003F',tz='Europe/Kiev')
    UA_BEI = Area_details(name='UA_BEI',code='10YUA-WEPS-----0',tz='Europe/Kiev')
    UA_DOBTPP = Area_details(name='UA_DOBTPP',code='10Y1001A1001A869',tz='Europe/Kiev')
    UA_IPS = Area_details(name='UA_IPS',code='10Y1001C--000182',tz='Europe/Kiev')
    UK = Area_details(name='UK',code='10Y1001A1001A92E',tz='Europe/London')
    XK = Area_details(name='XK',code='10Y1001C--00100H',tz='Europe/Rome')



from operator import __ge__ 
from operator import __le__ 
from typing import Callable

from dataclasses import dataclass,field
import operator
from .Sensors import *

class Comperators():
    greater_equals = __ge__
    lower_equas = __le__



@dataclass
class Feedback():
    sensor : Sensor
    target_value : float
    comperator : Callable[[float, float], bool]
    def polling(self):
        return self.comperator(self.sensor.value, self.target_value)
    








from typing import List, Tuple
from dataclasses import dataclass
import json
import base64

from config_Test import price_Logic1
from spotON_sdk import Market,Price_Logic,Continuous_On_Time,Interrupted_On_Time,Timeframes,Timeframe

@dataclass()
class SwitchPattern:
    area_code :str
    nr_of_hours_on :int
    pricefinding :dict
    resolution :float
    timeframes : List[int]


def convert_switch_pattern(price_Logic1: Price_Logic):
        
    area_code = price_Logic1.market.area_code # type: ignore
    nr_of_hours_on = price_Logic1.nr_of_hours_on
    pricefinding = price_Logic1.pricefinding.__dict__
    resolution = price_Logic1.resolution
    timeframes = price_Logic1.timeframes.possible_hours

    # Create and return a new SwitchPattern object
    return SwitchPattern(area_code, nr_of_hours_on, pricefinding, resolution, timeframes)

def create_escaped_switch_pattern_json(price_Logic:Price_Logic):
    switch_pattern = convert_switch_pattern(price_Logic)

    switch_pattern_json = json.dumps(switch_pattern.__dict__)
    escaped_switch_pattern_json = json.dumps(switch_pattern_json)

    return escaped_switch_pattern_json

def debug():
    print(create_escaped_switch_pattern_json(price_Logic1))


    
switch_pattern = convert_switch_pattern(price_Logic1)
pricefinding_Name = switch_pattern.pricefinding["name"]
if pricefinding_Name == "Continuous_On_Time":
    pricefinding = Continuous_On_Time(week=switch_pattern.pricefinding["week"],
                                      best_hour=switch_pattern.pricefinding["best_hour"])
elif pricefinding_Name == "Interrupted_On_Time":
    pricefinding = Interrupted_On_Time()
else:
    raise Exception("pricefinding_Name is not valid")

start = switch_pattern.timeframes[0]
end = switch_pattern.timeframes[-1]

'''price_Logic = Price_Logic(nr_of_hours_on=switch_pattern.nr_of_hours_on,
            market=switch_pattern.area_code,
            timeframes=Timeframes(Timeframe())'''

print(start,end)





from dataclasses import dataclass

from pydantic import root_validator


from typing import Optional,List


from .customBaseModel import CustomBaseModel


import pandas as pd
import pytz

class Country(CustomBaseModel):
    capital: str
    country_code: str
    country_name: str
    emoji: str = ""
    UTC_time_difference: pd.Timedelta = pd.Timedelta(hours=0)
    
    @classmethod
    def _validate_timezone(cls, values):
        try:
            timezone = f"Europe/{values['capital']}"
            local_tz = pytz.timezone(timezone)
        except:
            print(f"{values['capital']} not found in European capitals.")
            raise ValueError(f"{values['capital']} not found in European capitals.")
        return values
    
    @classmethod
    def _calculate_utc_time_difference(cls, values):
        timezone = f"Europe/{values['capital']}"
        local_tz = pytz.timezone(timezone)
        utc_dt = pd.Timestamp.utcnow().to_pydatetime()
        local_dt = utc_dt.astimezone(local_tz)
        local_dt_Offset = local_dt.utcoffset()
        values["UTC_time_difference"] = local_dt_Offset
        return values

    @classmethod
    def _country_code_to_emoji(cls, values):
        country_code = values.get('country_code')
    
        if "_" in country_code:
            cleaned_code = country_code.split("_")[0]
        else:
            cleaned_code = country_code
            
        OFFSET = 127397
        values["emoji"] = ''.join(chr(ord(c) + OFFSET) for c in cleaned_code.upper())
        return values

    @root_validator(pre=True)
    def validate_and_calculate(cls, values):
        values = cls._validate_timezone(values)
        values = cls._calculate_utc_time_difference(values)
        values = cls._country_code_to_emoji(values)
        return values



def get_country_by_code(countries: List[Country], country_code: str) -> Optional[Country]:
    """
    Returns the country object that matches the given country code.
    If no such object is found, returns None.
    """
    for country in countries:
        if country.country_code == country_code:
            return country
    return None



@dataclass
class all_Countries():
    Austria = Country(capital="Vienna", country_code="AT", country_name="Austria")
    Belgium = Country(capital="Brussels", country_code="BE", country_name="Belgium")
    Bulgaria = Country(capital="Sofia", country_code="BG", country_name="Bulgaria")
    Croatia = Country(capital="Zagreb", country_code="HR", country_name="Croatia")
    Cyprus = Country(capital="Nicosia", country_code="CY", country_name="Cyprus")
    Czech_republic = Country(capital="Prague", country_code="CZ", country_name="Czech republic")
    Denmark = Country(capital="Copenhagen", country_code="DK", country_name="Denmark")
    Estonia = Country(capital="Tallinn", country_code="EE", country_name="Estonia")
    Finland = Country(capital="Helsinki", country_code="FI", country_name="Finland")
    France = Country(capital="Paris", country_code="FR", country_name="France")
    Germany = Country(capital="Berlin", country_code="DE", country_name="Germany")
    Greece = Country(capital="Athens", country_code="GR", country_name="Greece")
    Hungary = Country(capital="Budapest", country_code="HU", country_name="Hungary")
    Ireland = Country(capital="Dublin", country_code="IE", country_name="Ireland")
    Italy = Country(capital="Rome", country_code="IT", country_name="Italy")
    Latvia = Country(capital="Riga", country_code="LV", country_name="Latvia")
    Lithuania = Country(capital="Vilnius", country_code="LT", country_name="Lithuania")
    Luxembourg = Country(capital="Luxembourg", country_code="LU", country_name="Luxembourg")
    Netherlands = Country(capital="Amsterdam", country_code="NL", country_name="Netherlands")
    Poland = Country(capital="Warsaw", country_code="PL", country_name="Poland")
    Portugal = Country(capital="Lisbon", country_code="PT", country_name="Portugal")
    Romania = Country(capital="Bucharest", country_code="RO", country_name="Romania")
    Slovakia = Country(capital="Bratislava", country_code="SK", country_name="Slovakia")
    Slovenia = Country(capital="Ljubljana", country_code="SI", country_name="Slovenia")
    Spain = Country(capital="Madrid", country_code="ES", country_name="Spain")
    Sweden = Country(capital="Stockholm", country_code="SE_1", country_name="Sweden")

    return_List= [Austria, Belgium, Bulgaria, Croatia, Cyprus, Czech_republic, Denmark, Estonia, Finland, France, Germany, Greece, Hungary, Ireland, Italy, Latvia, Lithuania, Luxembourg, Netherlands, Poland, Portugal, Romania, Slovakia, Slovenia, Spain, Sweden]

    @classmethod
    def get_country_by_name(cls,country_name: str) -> Optional[Country]:
        """
        Returns the country object that matches the given country name.
        If no such object is found, returns None.
        """
        for country in cls.return_List:
            if country.country_name == country_name:
                return country
        return None





from spotON_sdk import *
from pprint import pprint
from spotON_sdk.Logic.Output.Switchtypes import Switchtypes
from spotON_sdk.Logic.Price import Price_Logic
from operator import __ge__,__le__
from spotON_sdk.spotON_controller import Controller

temperature_sensor = Sensor(25,Units.temperature)
switch = Switchtypes.SWITCH
target_value = 27

@dataclass
class all_Configs():
    full_Day_Switch_Config = Price_Logic(nr_of_hours_on=24,
                                         market=Markets.austria,
                                         on_hours=On_Hours(timeframes_list=[Wholeday()]),
                                            pricefinding=Continuous_On_Time(week=1))



@dataclass
class all_Feedbacks():

    heater_feedback = Feedback(sensor=temperature_sensor,
                    target_value=target_value,
                    comperator=__ge__)

    humidifier_feedback = Feedback(sensor=temperature_sensor,
                        target_value=target_value,
                        comperator=__le__)

    dehumidifier_feedback = Feedback(sensor=temperature_sensor,
                        target_value=target_value,
                        comperator=__le__)

@dataclass
class all_Presets():

    heater_Preset = Controller(name="Heater",
                           config = all_Configs.full_Day_Switch_Config,
                           feedback=[all_Feedbacks.heater_feedback],
                           output_Switch=Switchtypes.SWITCH)
    humidity_Preset = Controller(name="Dehumidifier",
                           config = all_Configs.full_Day_Switch_Config,
                           feedback=[all_Feedbacks.humidifier_feedback],
                           output_Switch=Switchtypes.SWITCH)


pprint (all_Presets.heater_Preset)




from datetime import datetime,timedelta
import pytz 
import pandas as pd


def get_today_midnight(timezone_string: str)->pd.Timestamp:
    CET_TZ = pytz.timezone(timezone_string)

    # get the current date and time in Lisbon timezone
    now_local_tz = pd.Timestamp.now(tz=CET_TZ)

    # set the time to 0:00
    today_midnight = now_local_tz.floor('D')
    return today_midnight

def add_day_to_timestamp(today_timestamp:pd.Timestamp):
    calculate_tomorrow_timestamp = (today_timestamp + pd.DateOffset(days=1)).normalize()
    return calculate_tomorrow_timestamp

def getTime(timezone:str):
    '''Get the current time in UTC timezone'''
    now_utc = datetime.now(pytz.utc)

    # Convert the current time to your desired timezone
    desired_timezone = pytz.timezone(timezone)
    now_local = now_utc.astimezone(desired_timezone)
    return now_local




def n_days_ago_or_future_midnight(now_local, n,hour = 23):
    # Add or subtract n days from the current date
    n_days_ago_or_future = now_local + timedelta(days=n)

    # Create a new datetime object for n days ago or in the future at 0:00 (midnight)
    n_days_ago_or_future_midnight = datetime(n_days_ago_or_future.year, n_days_ago_or_future.month, n_days_ago_or_future.day, hour, 0)

    return n_days_ago_or_future_midnight








from typing import List

from .auth import Auth
from .light import Light


class ExampleHubAPI:
    """Class to communicate with the ExampleHub API."""

    def __init__(self, auth: Auth):
        """Initialize the API and store the auth so we can make requests."""
        self.auth = auth

    async def async_get_lights(self) -> List[Light]:
        """Return the lights."""
        resp = await self.auth.request("get", "lights")
        resp.raise_for_status()
        return [Light(light_data, self.auth) for light_data in await resp.json()]

    async def async_get_light(self, light_id) -> Light:
        """Return the lights."""
        resp = await self.auth.request("get", f"light/{light_id}")
        resp.raise_for_status()
        return Light(await resp.json(), self.auth)




import time
from pydantic import validator
from typing import List, Any

from .customBaseModel import CustomBaseModel


class Timeframe(CustomBaseModel):
    start: int
    end: int

    @validator('*')
    def validate_hours(cls, value):
        if not 0 <= value < 24:
            raise ValueError("hours should be in range 0-23")
        return value


class Timeframes(CustomBaseModel):
    timeframes: List[Timeframe] =  [Timeframe(start=0, end=23)]

    # Create a new constructor that accepts the start and end parameters directly
    def __init__(self, **data: Any) -> None:
        # We intercept the "start" and "end" arguments and use them to create a Timeframe
        start = data.pop('start', 0)  # Provide a default value if you want
        end = data.pop('end', 23)  # Provide a default value if you want
        timeframe = Timeframe(start=start, end=end)
        data['timeframes'] = [timeframe]
        super().__init__(**data)

    @property
    def possible_hours(self) -> List[int]:
        hours = set()
        for timeframe in self.timeframes:
            start, end = timeframe.start, timeframe.end
            end += 1
            if start <= end:
                hours.update(range(start, end))
            else:  # the timeframe goes over midnight
                hours.update(range(start, 24))
                hours.update(range(0, end))
        return sorted(list(hours))
    
    def possible_hours_with_utc_offset(self, utc_offset: int) -> List[int]:
        hours = set()
        for timeframe in self.timeframes:
            start = (timeframe.start + utc_offset) % 24
            end = (timeframe.end + utc_offset) % 24 + 1

            if start < end:
                hours.update(range(start, end))
            else:
                hours.update(range(start, 24))
                hours.update(range(0, end))

        return sorted(list(hours))


    def add_timeframe(self, start: int, end: int):
        self.timeframes.append(Timeframe(start=start, end=end))

    def add_timeframe_array(self, timeframe_array: List[Timeframe]):
        for timeframe in timeframe_array:
            self.add_timeframe(timeframe.start, timeframe.end)

        

    def remove_timeframe(self, start: int, end: int):
        self.timeframes = [timeframe for timeframe in self.timeframes if not (timeframe.start == start and timeframe.end == end)]
    
    def remove_initial_timeframe(self):
        self.timeframes = self.timeframes[1:]

    def set_whole_day(self):
        self.timeframes.append(Timeframe(start=0, end=23))

    def set_morning(self):
        self.timeframes.append(Timeframe(start=5, end=12))

    def set_afternoon(self):
        self.timeframes.append(Timeframe(start=12, end=17))

    def set_evening(self):
        self.timeframes.append(Timeframe(start=17, end=20))

    def set_night(self):
        self.timeframes.append(Timeframe(start=20, end=5))

    def as_String(self):
        timeframe_list = []
        for timeframe in self.timeframes:
            timeframe_list.append(str(timeframe.start) + " - " + str(timeframe.end) + " ")
        timeframe_str = ', '.join(map(str, timeframe_list))
        return timeframe_str

            

def hours_to_timeframes(hours_list: List[int]) -> Timeframes:
    timeframes = Timeframes()
    timeframes.remove_initial_timeframe()

    hours_list = sorted(set(hours_list))  # Ensure hours are unique and sorted

    start = None
    prev_hour = None

    for hour in hours_list:
        if start is None:
            start = hour  # Start of a new timeframe
        elif prev_hour is not None and hour != prev_hour + 1:
            # If there's a gap, we end the current timeframe and start a new one
            timeframes.add_timeframe(start=start, end=prev_hour)
            start = hour
        prev_hour = hour

    if start is not None and prev_hour is not None:
        # Close the last timeframe
        timeframes.add_timeframe(start=start, end=prev_hour)

    
    return timeframes





"""
The spotON_sdk package provides tools for interacting with the spotON API.

This package includes modules for authenticating with the API, sending requests,
and processing responses. It also includes utility functions for working with
the data returned by the API.
"""


from .Logic import *
from .data_helpers import *
from .API import *
from . import spotON_controller

__version__ = "0.0.5.58"
__all__ = [name for name in dir() if not name.startswith('_')] #type: ignore



from aiohttp import ClientSession, ClientResponse


class Auth:
    """Class to make authenticated requests."""

    def __init__(self, websession: ClientSession, host: str, access_token: str):
        """Initialize the auth."""
        self.websession = websession
        self.host = host
        self.access_token = access_token

    async def request(self, method: str, path: str, **kwargs) -> ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)
        
        headers = {'Content-type': 'application/json'} # Set the content type to JSON
        #headers["authorization"] = self.access_token
        print (f"{path}")
        return await self.websession.request(
            method, f"{self.host}/{path}", **kwargs, headers=headers,
        )
    
import asyncio
import aiohttp

async def main():
    async with aiohttp.ClientSession() as session:
        url = 'https://km28t5snp5.execute-api.eu-central-1.amazonaws.com/Prod/echo'
        url = 'http://127.0.0.1:8000'

        auth = Auth(session, url, "secret_access_token")

        # This will fetch data from http://example.com/api/lights
        resp = await auth.request("get", "lights")
        print("HTTP response status code", resp.status)
        print("HTTP response JSON content", await resp.json())



asyncio.run(main())


from locale import currency
from math import e
from re import T, escape

from numpy import cross
from .spotON_Areas import Currency, spotON_Area,Area_details
import pandas as pd
from dataclasses import dataclass, field
from typing import Optional, Dict,Any

from typing import List
from .countries import *
from .bidding_zones import bidding_zones
from ...data_helpers import get_today_midnight,add_day_to_timestamp

from .customBaseModel import CustomBaseModel

from pydantic import Field, root_validator,validate_model




class Market(CustomBaseModel):
    area: Area_details
    country: Country
    alias: Optional[str] = None
    cities: str = Field(default="")
    name: str = Field(default= "",init = False)

    @root_validator(pre=True)
    def set_codes(cls, values):
        country = values.get('country')
        if country and 'country_name' in country:
            values["name"] = country['country_name']
        return values
    
    @root_validator(pre=True)
    def set_name(cls, values):
        country :Country= values.get('country')
        area = values.get('area')
        if country and area:
            values["name"] = f"{country.emoji} {country.country_name} {area.name}"
        return values
    
    def get_start_end_hours_in_UTC(self) -> tuple[pd.Timestamp,pd.Timestamp]:
        starthour_local_tz = get_today_midnight(self.area.tz)
        endhour_local_tz= add_day_to_timestamp(starthour_local_tz)

        #convert to UTC
        starthour_utc = starthour_local_tz.tz_convert('UTC')
        endhour_utc = endhour_local_tz.tz_convert('UTC')
        endhour_utc = endhour_utc - pd.Timedelta(hours=1)
    
        return starthour_utc,endhour_utc

class MarketNotFoundError(ValueError):
    pass



class Markets():

    austria = Market(area=spotON_Area.AT.value,country=all_Countries.Austria)
    belgium = Market(area=spotON_Area.BE.value,country=all_Countries.Belgium)
    bulgaria = Market(area=spotON_Area.BG.value,country=all_Countries.Bulgaria)
    croatia = Market(area=spotON_Area.HR.value,country=all_Countries.Croatia)
    czech_republic = Market(area=spotON_Area.CZ.value,country=all_Countries.Czech_republic)
    #denmark = Market(area=spotON_Area.DK.value,country=all_Countries.Denmark)
    estoonia = Market(area=spotON_Area.EE.value,country=all_Countries.Estonia)
    finland = Market(area=spotON_Area.FI.value,country=all_Countries.Finland)
    france = Market(area=spotON_Area.FR.value,country=all_Countries.France)
    germany = Market(area=spotON_Area.DE_LU.value,country=all_Countries.Germany,alias="DE_LU")
    greece = Market(area=spotON_Area.GR.value,country=all_Countries.Greece)
    hungary = Market(area=spotON_Area.HU.value,country=all_Countries.Hungary)
    #ireland = Market(area=spotON_Area.IE_SEM.value,country=all_Countries.Ireland)
    #italy = Market(area=spotON_Area.IT.value,country=all_Countries.Italy)
    latvia = Market(area=spotON_Area.LV.value,country=all_Countries.Latvia)
    lithuania = Market(area=spotON_Area.LT.value,country=all_Countries.Lithuania)
    luxembourg = Market(area=spotON_Area.DE_LU.value,country=all_Countries.Luxembourg,alias="DE_LU")
    netherlands = Market(area=spotON_Area.NL.value,country=all_Countries.Netherlands)
    poland = Market(area=spotON_Area.PL.value,country=all_Countries.Poland)
    portugal = Market(area=spotON_Area.PT.value,country=all_Countries.Portugal)
    romania = Market(area=spotON_Area.RO.value,country=all_Countries.Romania)
    slovakia = Market(area=spotON_Area.SK.value,country=all_Countries.Slovakia)
    slovenia = Market(area=spotON_Area.SI.value,country=all_Countries.Slovenia)
    spain = Market(area=spotON_Area.ES.value,country=all_Countries.Spain)
    #sweden1 = Market(area=spotON_Area.SE_1.value,country=all_Countries.Sweden)
    #luxembourg = Market(Area.DE_LU,Luxembourg)

    #sweden2 = Market(Area.SE_2,Sweden)
    #sweden3 = Market(Area.SE_3,Sweden)
    #sweden4 = Market(Area.SE_4,Sweden)
    markets_List = [value for key, value in vars().items() if isinstance(value, Market)]
    def __init__(self,debug = True):

        if debug:
            self.markets_List = [self.austria,self.germany]        

    @staticmethod
    def get_market_by_name(name: str) -> Optional[Market]:
        for market in Markets.markets_List:
            if market.name == name: # type: ignore
                return market

        raise MarketNotFoundError

    @staticmethod
    def get_market_by_code(area_code: str) -> Optional[Market]:
        for market in Markets.markets_List:
            #print (f"Try to find {area_code =} in {market}")
            if market.country.country_code == area_code or market.alias == area_code:
                return market

        raise MarketNotFoundError





